 /******************************************************************************
 *                         VAULT INFORMATION SERVICES                         *
 *                          TEXAS INSTRUMENTS, INC.                           *
 *----------------------------------------------------------------------------*
 *                  "TUSB3210 USB to I2C, SPI, One-Wire and GPIO"             *
 *                  "Based on the TUSB2136 Generic Keyboard Demo Program"     *
 *                                                                            * 
 * Programmed by: Art Kay, Senior Apps (kay_art@ti.com)                  	  * 
 * Based on software by: 													  *
 *						 Craig Steiner, VIS (csteiner@vaultbbs.com)           * 
 *  					 Sumithra Ramanathan, VIS (sumithra_ramanathan@ti.com)* 
 *						 Jim Chen											  *
 * Develop Date:  2/16/2006                                                   *  
 *----------------------------------------------------------------------------*
 * Description: This software takes 32 byte words from the PC host and uses   *
 *	the 32 bytes to generate a digital I/O.  Interfaces supported are SPI, 	  *
 *  I2C, One-Wire, and GPIO.  The software also allows control of switched    *
 *  power supplys, a quad 16 bit DAC, and two 16 bit ADC's.  This software    * 
 *  is based on a keyboad demo, and consequently you may see some remnents    *
 *  of this software.														  *
 *                                                                            *
 *                                                                            *
 ******************************************************************************/

//#include <io51.h>
#include <reg52.h>
#include "types.h"
#include "delay.h"
#include "usb.h"
#include "tusb2136.h"
#include "locports.h>

// Maximum size of the Keypress buffer is 6 bytes, because that's what fits in
// a single HID standard keyboard response

#define KEYPRESS_BUFFER_SIZE			0x06

// The following definitions indicate which bit of the data byte sent from
// the host to the keyboard correspond to each of the given LEDs.

#define BIT_NUMLOCK					0x01
#define BIT_CAPSLOCK				0x02
#define BIT_SCROLLLOCK				0x04

// The following definitions indicate which GPIO pin corresponds to each
// of the keyboard LEDs.

#define PIN_NUMLOCK				    P3_4
#define PIN_CAPSLOCK				P3_3
#define PIN_SCROLLLOCK				P3_2

extern unsigned char s0123;
extern BYTE deviceReady;
extern BYTE bSuspended;

extern void gpioInitialization(void);
extern void setLEDs(BYTE bData);
extern int readSwitches(void);
extern BYTE mystrlen(char *string);
void sendUartByte(BYTE bValue);
void sendUartString(char *string);
void sendUartHex(BYTE bValue);
void ReceiveUartByte(BYTE *bValue);



extern BYTE mymemcmp(char *string1, char *string2, BYTE count);
extern void mymemcpy(char *string1, char *string2, BYTE count);
extern void InitializeUsbData(void);
extern void InitializeUsbHub(void);
extern void InitializeUsbFunction(void);
extern void GetVidPidSetting(void);
unsigned char intFlags = 0x00;
unsigned char otherFlags = 0x00;

//void ClearKeyBoardBuffer(void);
void sendKeyData(int x);
void UpdateIEP1WithKeypress(void);
BYTE old_P0; // save status of Port 0
BYTE old_P1; // save status of Port 1
BYTE old_CTRL2;
BYTE USB_Called_Suspend;

BYTE bLED[32];
BYTE modifierByte = 0x00;
BYTE keypressBuffer[KEYPRESS_BUFFER_SIZE + 1];
char previousIEP1packet[8];
char *iep1Buffer = 0x00;
BOOL bEcho = FALSE;
//#pragma memory = dataseg(TUSB2136_DEBOUNCE_SEG)
xdata BYTE keypressDebounce[256] _at_ 0xfd80;
//#pragma memory = default

void delay(int i)
{
  while(i--);
}

void main(void)
{
	int vidPidMask;
	int iTemp;
	unsigned char bEchoChar = 0xFF;
#ifdef SERIALDEBUG
	// Enable serial port for debug at 9600 baud, using 48.000MHZ.
	SCON = 0x52; // TEMP AAK
	TH1 = -13;
	TMOD = 0x20;
	TR1 = 1;
	TI=1;
  	sendUartString("---------------------------------------------\r\n");
    sendUartString("---TUSB3210----------------------------------\r\n");
 	sendUartString("---Just Booted the USB Chip -----------------\r\n");
 	sendUartString("---Software Rev: 2.0,  Oct 13, 2010 ----------\r\n"); // note change revision in USB.C, WriteToDut2 case 7 also
 	sendUartString("---Hardware Rev: Super-Mini Dig Platform Rev A-\r\n");
 	sendUartString("---------------------------------------------\r\n");
 	sendUartString(" \r\n");
	// rev 1.1 to make "wake up" work so that it does not corrupt reg 0
	// rev 1.1 clean up rs232 writes
	// rev 1.2 fix one disconnect open one, open pull up
	// REV 1.3 change pid.  old pid = 0x1234 - 0x1237, new pid = 0x2F90 -  0x2F93
//	sendUartString(" \r\n");
//	sendUartString(" \r\n");
//	sendUartString(" \r\n");

#endif

#if 0
	while(!bEcho)
	{
		ReceiveUartByte(&bEchoChar);
		sendUartByte(bEchoChar);
		if(bEchoChar != 0xFF)
			bEcho = TRUE;
	}
#endif

	bPUR0 = 0x00;   // enable internal pull-ups
	bPUR1 = 0x00;
	bPUR2 = 0x00;
	bPUR3 = 0x00;
	
	//Ports 0-3 power up in the high state ~CW
	P3 = 0xff; // speed-up pull-ups
	P2 = 0xff;
	P1 = 0xff;
	P0 = 0xff;
	
	old_P0=0xff; // initial state of ports
	old_P1=0xff; // initial state of ports
	old_CTRL2=0xff;// initial ctrl2


	//Sets bit 2.0 low and bits 2.1 and 2.2 high
	//This will set the output of the TPS2115A to "Hi-Z"  ~CW

		P2=(old_CTRL2&0xFE)|0x06; 
		//P3=0xFF;
		//P3=0xEF;
		//P3=0xFF;
		old_CTRL2=P2;
		//sendUartString("POWER OFF \r\n");


	//Sets bits 2.5 and 2.6 low
	//This will set both "DIR" bits on the "1T45" level translators
	//to drive from "B -> A" making the GPIO inputs on power-up ~CW

		P2=(old_CTRL2&0x9F); 
		//P3=0xFF;
		//P3=0xEF;
		//P3=0xFF;
		old_CTRL2=P2;
		//sendUartString("POWER OFF \r\n");
	
	//Sets all bits of P2 to "0"
	//This will enable the TPS2115 and set the output to +3.3V 
	//I don't think we need this anymore and am commenting it out ~CW 

		//P2=0; 
		//P3=0xFF;
		//P3=0xDF;
		//P3=0xFF;
		//sendUartString("MUX Write!\r\n");
	

	//Sets bit 6 of P2 to high
	//This was used in the USB-DAQ to set the "EN_RS232" signal high
	//On Super-Mini-Dig this will set the "DIR2" signal high 
	//This causes the U9 level-shifter to move signals from the A-->B bus
	//I don't think we need this anymore and am commenting it out         ~CW

		//P2=(old_CTRL2)|0x40;
		//P3=0xFF;
		//P3=0xEF;
		//P3=0xFF;
		//old_CTRL2=P2;
		//sendUartString("RS232 \r\n");

	//Sets all bits of Port 2 to "1"
	//This is redundant from above now and I'm removing it     ~CW

		//P2=0xFF; 
		//P3=0xFF;
		//P3=0xBF;
		//P3=0xFF;
		//sendUartString("Init ctrl = FF \r\n");


/*
	// ENABLE RS232
	P2=(old_CTRL2&0xFE)|0x06; //DUT POWER OFF
	P3=0xFF;
	P3=0xEF;
	P3=0xFF;
	old_CTRL2=P2;
	USB_Called_Suspend=0;
		  	sendUartString("INIT DUT POWER OFF\r\n");
	// end of ENABLE RS232

	// Set inital dut power off
	P2=(old_CTRL2&0xFE)|0x06; //DUT POWER OFF
	P3=0xFF;
	P3=0xEF;
	P3=0xFF;
	old_CTRL2=P2;
	USB_Called_Suspend=0;
		  	sendUartString("INIT DUT POWER OFF\r\n");
	// end of Set inital dut power off

		// Set ads1100 cal mux
	P2=0x00; //DUT POWER OFF
	P3=0xFF;
	P3=0xED;
	P3=0xFF;
	//old_CTRL2=P2;
	USB_Called_Suspend=0;
	// end of Set ads1100 cal mux

		P2=0x00; 
		P3=0xFF;
		P3=0xDF;
		P3=0xFF;

*/



	GetVidPidSetting(); // Obtain the S0-S3 setting to determine VID/PID
	InitializeUsbHub(); // Initialize the USB hub connecting it to upstream port
	InitializeUsbFunction(); // Initialize the USB embedded function
//	ClearKeyBoardBuffer(); // Clear the keyboard buffer
	
	vidPidMask = 1 << s0123; // Prepares our VID/PID mask to decide whether a given key is valid
	while(!deviceReady); // Wait for the setup packets to be exchanged, etc.
	
	for(iTemp = 0; iTemp < 256; iTemp++)
		keypressDebounce[iTemp] = 0x00;
 
	while(1) { // Indefinite loop to handle keyboard scanning
	  
	  if(bSuspended & USB_Called_Suspend) {
		  	sendUartString("I am suspended\r\n");
 
		 sendUartString("USB_Called_Suspend = ");
 	 	 sendUartHex(USB_Called_Suspend);
 	 	 sendUartString("   \r\n");
/*
	        // disable P0, P1 and part of P3 internal pull-ups
			bPUR0 = 0xFF;
			bPUR1 = 0xFF;
			bPUR3 = 0xc3;  // disable P3.6, P3.7, P3.0, & P3.1 pull-ups 
			
			P0 = 0x00;
			P1 = 0x00;
			P3 = 0xff;     // turn LED off 
			P3_6 = 0;
			P3_7 = 0;                                                      

            // TUSB3210 Base EVM needs follow codes, otherwise it will comsume more 100uA power
            P3_0 = 0;      // since dip switch may tie P3.0 & P3.1 to ground
            P3_1 = 0;      
			
              EX1 = ENABLE;  // enable P2 interrupt  
			// Set IDLE bit in PCON to bring MCU to suspend
			 PCON |= 0x01;

            // uC wakeups from suspend
			EX1 = DISABLE;
			bPUR0 = 0x00;
			bPUR1 = 0x00;
			bPUR3 = 0x00;

			// Execution will stop until we wake back up, at which point
			// we then conitnue by indicating we're no longer in Suspend
			// mode.
*/
					  	sendUartString("I am leaving suspended!!!\r\n");

			bSuspended = FALSE;
					  	sendUartString("I am out of suspended???\r\n");
 
		} // end of if(bSuspended)
		 	    
		if(!deviceReady)
			continue;
	} // end of while(1)
	
}

/***************************************************************************
 * Function:   ClearKeyBoardBuffer                                         *
 * Programmer: Craig Steiner (csteiner@vaultbbs.com)                       *
 * Description: Clears the keyboard's internal keypress buffer.  The       *
 *    keypress buffer is 6 bytes which represent a maximum of 6 keypresses *
 *    that can be returned via USB.  Once the keypresses are sent via USB, *
 *    this routine may be called to clear the buffer.                      * 
 ***************************************************************************/
/*
void ClearKeyBoardBuffer(void)
{
	BYTE bTemp;

	for(bTemp = 0; bTemp < KEYPRESS_BUFFER_SIZE; bTemp++)
		keypressBuffer[bTemp] = 0x00;
}
*/
/***************************************************************************
 * Function:   UpdateIEP1WithKeypress                                      *
 * Programmer: Craig Steiner (csteiner@vaultbbs.com)                       *
 * Description: Is called by the main() program in order to send the       *
 *    contents of the keypressBuffer[] to the host via USB.  The function  *
 *    does the following:                                                  *
 *                                                                         * 
 *    1. Makes sure the current IEP1 Byte Counter is "NAK", which means    * 
 *       there is nothing pending to send.  Any other value means the last * 
 *       packet sent to the host hasn't been sent yet, so we don't send    * 
 *       anything else until the previous packet is sent.                  * 
 *    2. If nothing is pending to send, the routine constructs the output  * 
 *       data packet in the IEP1 Output buffer, copying the modifier byte, * 
 *       the constant 0x00 byte, and the 6 bytes of keypressBufferp[] data.*   
 *    3. Compares to see if the current packet that is ready to send is    * 
 *       the same as the last packet that was sent.  If it is, it exits    * 
 *       since only *changes* in the keyboard state need to be reported.   *   
 *    4. If all of the above tests are successful, the routine sets the    * 
 *       Byte Counter to 8 which will cause the 8-byte packet to be        * 
 *       sent when the next IEP1 poll is received from the host.           *   
 ***************************************************************************/
/*
void UpdateIEP1WithKeypress(void)
{
	BYTE bTemp;

	// We first check to see if the character count is NAK.  If it's not
	// NAK that means we still have some output pending, so we leave it
	// alone for now.
    sendUartString("we don't go IN UpdateIEP1WithKeypress(void), DO WE ? \r\n");

	if(bIEPDCTX_1 != EPBCT_NAK)
		{
		ClearKeyBoardBuffer();
		return;
		}

	// Start by initialization the HID Keyboard report header
	iep1Buffer[0] = modifierByte; // Holds shift, alt, etc.
	iep1Buffer[1] = 0x00; // Second byte is always 0x00
	
	// We now copy the keypress buffer into the rest of the iep1Buffer.
	for(bTemp = 0; bTemp < 6; bTemp++)
		iep1Buffer[2 + bTemp] = keypressBuffer[bTemp];

	// We now compare the currently prepared IEP1 output buffer with the last
	// packet that was sent, which is held in the variable previousIEP1packet.
	// If the current packet is the same as the previous packet sent, we 
	// don't need to do anything since the host will assume the previous state
	// is still current until we report a different state.
	if( mymemcmp(iep1Buffer, previousIEP1packet, 8) == 0)
		{
		ClearKeyBoardBuffer();
		return;
		}

	// We now check to see if the device is currently suspended.  If it is,
	// the keypress should wake the system up--so we generate a Wakeup
	// pulse so things come back to life.
#ifdef XXX
	if(bSuspended)
		bUSBCTL |= USBCTL_RWUP;
#endif
	// We finally the set the CTX_1 to the number of bytes in the IEP1 output
	// buffer (8).  Thus, the next time the host issues an IN1 poll to the
	// TUSB2136, the data will be sent out immediately.  We will know that
	// the transfer has taken place when the IEP1InterruptHandler() interrupt
	// handler is called.
	bIEPDCTX_1 = 8;
}
*/
/***************************************************************************
 * Function:   IEP1InterruptHandler                                        *
 * Programmer: Craig Steiner (csteiner@vaultbbs.com)                       *
 * Description: This function is the IEP1 interrupt handler.  It is called *
 *    whenever a packet of data has been transmitted from the device to the*
 *    host.  The routine performs the following tasks:                     *
 *                                                                         * 
 *    1. Makes a copy of the data packet that was just sent in the         * 
 *       variable previousIEP1packet.  This is used by the                 *
 *       UpdateIEP1WithKeypress function to determine whether or not there *
 *       has been a change in the keyboard state.                          *   
 *    2. Clears the keypressBuffer to allow the main program loop to       *  
 *       scan the keyboard and generate a new packet.                      *  
 *                                                                         *     
 ***************************************************************************/

#ifndef TI_HPL_TESTS
void IEP1InterruptHandler(void)
{
	// This interrupt is triggered when we've finished sending a block of data
	// to the host.  At that point we copy the current IEP1 buffer to the
	// "previousIEP1packet" buffer.  This holds a copy of the last data that
	// was sent to the host.  This is used later to see if a subsequent packet
	// is the same as a new packet.  If it is, we won't have to resend it.

	mymemcpy(previousIEP1packet,  iep1Buffer, 8);
//	BYTE str;
//
//	for(str = 0; str < 8; str++)
//		previousIEP1packet[str] = iep1Buffer[str];
	
	// We clear the keypressBuffer.  The keyboard scan process will only begin
	// if the keypressBuffer is clear.
//	ClearKeyBoardBuffer();
}
#else
void IEP1InterruptHandler(void)
{
//	BYTE i;
//	for(i=0;i<32;i++)
//	{
	//	bufIEP1Buffer[i]=pbXBufferAddress[i]+1;
//		sendUartHex(pbXBufferAddress[i]);
//		sendUartString("  ");
//	}

	//sendUartString("ti_hpl_tests IEP1InterruptHandler\r\n");
	// Just clear the NAK bit
	bIEPDCTX_1 = EPBCT_NAK ;
}
#endif

/***************************************************************************
 * Function:   OEP0SetLEDs                                                 *
 * Programmer: Craig Steiner (csteiner@vaultbbs.com)                       *
 * Description: This function is called by the usbReceiveNextPacketOnOEP0  *
 *    routine when the data packet of a LED SetReport transaction is       *
 *    received.  This routine sets/clears the keyboard LEDs based on the   *
 *    data received from the host.                                         * 
 *                                                                         *     
 ***************************************************************************/

void OEP0SetLEDs(void)
{
// This code is only enabled if this is a "real" keyboard, not the
// demo version selected by DIPSWITCHES.  If it is the demo version,
// changing the values of the P3.2-P3.4 pins seems to cause problems.
/*
	BYTE i;
	for(i=0;i<8;i++)
	{
		sendUartHex(bLED[i]);
		sendUartString("  ");
	}
	sendUartString("\r\n");

	if(bLED[0] & BIT_NUMLOCK)
		PIN_NUMLOCK = 0;
	else
		PIN_NUMLOCK = 1;

	if(bLED[0] & BIT_CAPSLOCK)
		PIN_CAPSLOCK = 0;
	else
		PIN_CAPSLOCK = 1;

	if(bLED[0] & BIT_SCROLLLOCK)
		PIN_SCROLLLOCK = 0;
	else
		PIN_SCROLLLOCK = 1;
*/
}
